<?php

declare(strict_types=1);

namespace Drupal\billwerk_subscriptions_manage\Controller;

use Drupal\billwerk_subscriptions\Subscriber;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockManagerInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Utility\Error;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for Billwerk Subscriptions SubscriptionJS routes.
 */
final class BillwerkSubscriptionsManageController extends ControllerBase {

  /**
   * The controller constructor.
   */
  public function __construct(
    private readonly BlockManagerInterface $pluginManagerBlock,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('plugin.manager.block'),
    );
  }

  /**
   *
   */
  public function currentUserSubscription(): array {
    try {
      // We use the existing block, no need to reinvent the wheel:
      $config = $this->config('billwerk_subscriptions_manage.settings');
      $plugin_block = $this->pluginManagerBlock->createInstance('billwerk_subscriptions_manage_billwerk_user_subscriptions', [
        'source' => $config->get('page.settings.source'),
        'show_active_subscription' => $config->get('page.settings.show_active_subscription'),
        'show_active_components' => $config->get('page.settings.show_active_components'),
        'no_active_components_text' => $config->get('page.settings.no_active_components_text'),
        'show_change_plan_variant_button' => $config->get('page.settings.show_change_plan_variant_button'),
        'show_cancel_plan_variant_button' => $config->get('page.settings.show_cancel_plan_variant_button'),
        'show_components_subscribe_button' => $config->get('page.settings.show_components_subscribe_button'),
        'show_components_unsubscribe_button' => $config->get('page.settings.show_components_unsubscribe_button'),
        'modal_actions' => $config->get('page.settings.modal_actions'),
      ]);
      $build = $plugin_block->build();
    }
    catch (\Exception $e) {
      // Log exception:
      Error::logException(\Drupal::logger('billwerk_subscriptions_manage'), $e);
      // Show error message:
      // @improve once https://www.drupal.org/project/drupal/issues/3439618 is fixed:
      $build = [
        'message' => [
          '#theme' => 'status_messages',
          '#message_list' => [
            'error' => [
              $this->t('The website encountered an unexpected error. Try again later.'),
            ],
          ],
          '#status_headings' => [
            'error' => $this
              ->t('Error message'),
          ],
        ],
      ];
    }

    return $build;
  }

  /**
   *
   */
  public function finished(): array {
    $build['content'] = [
      '#type' => 'item',
      '#markup' => $this->t('Thank you for your order!'),
      '#attached' => [
        'library' => 'billwerk_subscriptions_manage/billwerk_subscriptions_manage',
        'drupalSettings' => [
          'billwerk_context' => 'finalize',
        ],
      ],
    ];

    return $build;
  }

  /**
   * Checks access for a specific request.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Run access checks for this account.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(AccountInterface $account) {
    try {
      $subscriber = Subscriber::loadByDrupalUid((int) \Drupal::currentUser()->id());
    }
    catch (\Exception $e) {
      $logHelper = \Drupal::service('billwerk_subscriptions.log_helper');
      $logHelper->logException($e);
      return AccessResult::forbidden($e->getMessage());
    }

    return AccessResult::allowedIf($subscriber->hasUserBillwerkContractId())
      ->andIf(AccessResult::allowedIfHasPermissions($account, ['billwerk_subscriptions_manage_change_own_subscription', 'billwerk_subscriptions_manage_cancel_own_subscription'], 'OR'));
  }

}
