<?php

declare(strict_types=1);

namespace Drupal\billwerk_subscriptions_manage\Form;

use Drupal\billwerk_subscriptions\Exception\EnvironmentException;
use Drupal\billwerk_subscriptions\Exception\SubscriberException;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a Billwerk Subscriptions Manage form.
 */
final class CancelSubscriptionForm extends ChangePlanVariantForm {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'billwerk_subscriptions_manage_cancel_subscription';
  }

  /**
   * {@inheritdoc}
   */
  public function buildSelectionForm(array $form, FormStateInterface $form_state): array {
    $this->buildCancelationMessage($form, $form_state);

    $form['actions']['#type'] = 'actions';
    $form['actions']['proceed'] = [
      '#type' => 'submit',
      '#value' => $this->t('Next'),
      '#validate' => ['::validateSelectionForm'],
      '#submit' => ['::proceedToConfirm'],
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   *
   */
  public function validateSelectionForm(array &$form, FormStateInterface $form_state) {
    $activePlanVariant = $this->subscriberBillwerkEntitiesHelper->getActiveBillwerkPlanVariant();
    $selectedPlanVariant = $form_state->getValue('planvariant');

    if ($activePlanVariant === $selectedPlanVariant) {
      $form_state->setErrorByName('planvariant', $this->t('You did not change the plan variant.'));
    }
  }

  /**
   *
   */
  protected function buildConfirmForm(array $form, FormStateInterface $form_state): array {
    // @todo
    $config = $this->configFactory()->get('billwerk_subscriptions_manage.settings');
    $cancelationMethod = $config->get('cancelation_method');
    if ($cancelationMethod === SettingsForm::CANCELATION_METHOD_DOWNGRADE) {
      if ($this->environment->isProduction()) {
        $newPlanVariantId = $config->get('billwerk_free_plan_variant_id_production');
      }
      elseif ($this->environment->isSandbox()) {
        $newPlanVariantId = $config->get('billwerk_free_plan_variant_id_sandbox');
      }
      else {
        throw new EnvironmentException('Unknown environment: ' . $this->environment->getEnvironmentName());
      }
      if (empty($newPlanVariantId)) {
        throw new SubscriberException('Registration plan variant for environment: ' . $this->environment->getEnvironmentName() . ' is empty. Order creation was not possible. Check your settings.');
      }

      $endComponentSubscriptionIds = [];
      $activeBillwerkComponentSubscriptions = $this->subscriber->getBillwerkContract()->getBillwerkContractSubscription()->getComponentSubscriptions();
      foreach ($activeBillwerkComponentSubscriptions as $activeBillwerkComponentSubscriptions) {
        $endComponentSubscriptionIds[] = $activeBillwerkComponentSubscriptions->getId();
      }

      // Downgrade to basic free subscription:
      $billwerkOrderObj = $this->subscriber->billwerkChangeSubscription(
      $newPlanVariantId,
      [],
      $endComponentSubscriptionIds,
        // No coupon:
      '',
        // Cancelation should happen at the end of the contract period:
        FALSE,
        // We need a committable (non-preview) order:
        FALSE,
        // We commit this order by JavaScript:
        FALSE,
      );
      $form_state->set('billwerkOrder', $billwerkOrderObj);
      $form_state->set('step', 'confirm');

      $this->buildOrderTableFormElement($form, $form_state, $billwerkOrderObj);
      $this->buildDowngradeOrderTermsFormElement($form, $form_state);

    }
    elseif ($cancelationMethod === SettingsForm::CANCELATION_METHOD_CANCEL_CONTRACT) {
      // @todo Implement if needed
    }
    $this->buildCancelationMessage($form, $form_state);
    $this->buildOrderActionsFormElement($form, $form_state, $this->t('Confirm subscription cancelation'));

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    // @todo We could commit the order server-side here, because no payment data is needed!
    dsm($form_state->get('billwerkOrder'));
    $billwerkOrderObj = $form_state->get('billwerkOrder');
    if (!empty($billwerkOrderObj) && !empty($billwerkOrderObj->Id)) {
      $response = $this->api->commitOrder($billwerkOrderObj->Id, []);
      $this->messenger()->addStatus($this->t('Your subscription has been canceled to the end of the contract.'));
      $form_state->setRedirect('billwerk_subscriptions_manage.current_user_subscription');
    }
    else {
      throw new SubscriberException('Order was not committed, because order id could not be determined.');
    }
  }

  /**
   *
   */
  protected function buildCancelationMessage(array &$form, FormStateInterface $form_state): void {
    $config = $this->configFactory()->get('billwerk_subscriptions_manage.settings');
    $cancelationMethod = $config->get('cancelation_method');
    if ($cancelationMethod === SettingsForm::CANCELATION_METHOD_DOWNGRADE) {
      $form['message'] = [
        'message' => [
          '#type' => 'inline_template',
          '#template' => '<div class="messages messages--status"><div class="message">{{ "Your subscription will be downgraded to the free basic plan at the end of the contract period.\nProceed to review."|t }}</div></div>',
          '#context' => [],
        ],
      ];
    }
    elseif ($cancelationMethod === SettingsForm::CANCELATION_METHOD_CANCEL_CONTRACT) {
      $form['message'] = [
        'message' => [
          '#type' => 'inline_template',
          '#template' => '<div class="messages messages--status"><div class="message">{{ "Your subscription will be canceled at the end of the contract period.\nProceed to review."|t }}</div></div>',
          '#context' => [],
        ],
      ];
    }
    else {
      throw new \UnexpectedValueException("Unexpected cancelation method: \"{$cancelationMethod}\"");
    }
  }

}
